@@ -33,9 +33,11 @@
 #if GST_CHECK_VERSION(1, 0, 0)
 #include <gst/video/videooverlay.h>
 #include <gst/video/navigation.h>
+#include <gst/audio/streamvolume.h>
 #else
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/navigation.h>
+#include <gst/interfaces/streamvolume.h>
 #endif
 
 #include <gst/pbutils/missing-plugins.h>
@@ -66,8 +68,7 @@ static void	parole_gst_play_file_internal 	(ParoleGst *gst);
 static void     parole_gst_change_state 	(ParoleGst *gst, 
 						 GstState new);
 
-static void	parole_gst_terminate_internal   (ParoleGst *gst, 
-						 gboolean fade_sound);
+static void	parole_gst_terminate_internal   (ParoleGst *gst);
 						 
 static GdkPixbuf * parole_gst_tag_list_get_cover_external (ParoleGst *gst);
 
@@ -103,6 +104,7 @@ struct ParoleGstPrivate
 {
     GstElement	 *playbin;
     GstElement   *video_sink;
+    GstElement   *audio_sink;
 
     GstBus       *bus;
     
@@ -128,6 +130,8 @@ struct ParoleGstPrivate
     gboolean      buffering;
     gboolean      update_color_balance;
     
+    gdouble	  volume;
+    
     gboolean      use_custom_subtitles;
     gchar*        custom_subtitles;
     
@@ -160,6 +164,7 @@ enum
 enum
 {
     PROP_0,
+    PROP_VOLUME,
     PROP_CONF_OBJ,
     PROP_ENABLE_TAGS
 };
@@ -1573,22 +1578,36 @@ parole_gst_missing_codec_dialog(ParoleGst *gst, GstMessage *msg)
     dialog = GTK_MESSAGE_DIALOG(gtk_message_dialog_new_with_markup(
                             NULL,
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+#if defined(__linux__)
                             GTK_MESSAGE_QUESTION,
+#elif defined(__FreeBSD__)
+                            GTK_MESSAGE_WARNING,
+#endif
                             GTK_BUTTONS_NONE,
                             "<b><big>%s</big></b>", 
                             _("Additional software is required.")
                             ));
                             
+#if defined(__linux__)
     gtk_dialog_add_buttons( GTK_DIALOG(dialog), 
                             _("Don't Install"),
                             GTK_RESPONSE_REJECT,
                             _("Install"), 
                             GTK_RESPONSE_ACCEPT,
                             NULL );
+#elif defined(__FreeBSD__)
+    gtk_dialog_add_button( GTK_DIALOG(dialog), 
+                            GTK_STOCK_OK,
+                            GTK_RESPONSE_ACCEPT);
+#endif
     
     gtk_message_dialog_format_secondary_markup(dialog,
+#if defined(__linux__)
                                              "Parole needs <b>%s</b> to play this file.\n"
                                              "It can be installed automatically.",
+#elif defined(__FreeBSD__)
+                                             "Parole needs <b>%s</b> to play this file.",
+#endif
                                              desc);
     
     return GTK_DIALOG(dialog);
@@ -1712,7 +1731,7 @@ parole_gst_bus_event (GstBus *bus, GstMessage *msg, gpointer data)
 #endif
 	             gst_install_plugins_context_free(ctx);
 	        }
-	        else if ( response == GTK_RESPONSE_REJECT )
+	        else if ( (response == GTK_RESPONSE_REJECT) || (response == GTK_RESPONSE_OK) )
 	            gtk_widget_destroy(GTK_WIDGET(dialog));
         }
 	    break;
@@ -2011,7 +2030,7 @@ parole_gst_check_state_change_timeout (gpointer data)
 	    
 	if ( ret_val )
 	{
-	    parole_gst_terminate_internal (gst, FALSE);
+	    parole_gst_terminate_internal (gst);
 	    gst->priv->state_change_id = 0;
 	    return FALSE;
 	}
@@ -2020,7 +2039,7 @@ parole_gst_check_state_change_timeout (gpointer data)
 }
 
 static void
-parole_gst_terminate_internal (ParoleGst *gst, gboolean fade_sound)
+parole_gst_terminate_internal (ParoleGst *gst)
 {
     gboolean playing_video;
     
@@ -2035,26 +2054,6 @@ parole_gst_terminate_internal (ParoleGst *gst, gboolean fade_sound)
 
     parole_window_busy_cursor (GTK_WIDGET (gst)->window);
     
-    if ( fade_sound && gst->priv->state == GST_STATE_PLAYING && !playing_video )
-    {
-	gdouble volume;
-	gdouble step;
-	volume = parole_gst_get_volume (gst);
-	/*
-	 * Fade-out on exit.
-	 */
-	if ( volume != 0 )
-	{
-	    while ( volume > 0 )
-	    {
-		step = volume - volume / 10;
-		parole_gst_set_volume (gst, step < 0.01 ? 0 : step);
-		volume = parole_gst_get_volume (gst);
-		g_usleep (15000);
-	    }
-	}
-    }
-    
     parole_gst_change_state (gst, GST_STATE_NULL);
 }
 
@@ -2114,6 +2113,9 @@ static void parole_gst_get_property (GObject *object,
     
     switch (prop_id)
     {
+	case PROP_VOLUME:
+	    g_value_set_double (value, gst->priv->volume);
+	    break;
 	case PROP_CONF_OBJ:
 	    g_value_set_pointer (value, gst->priv->conf);
 	    break;
@@ -2140,6 +2142,9 @@ static void parole_gst_set_property (GObject *object,
 	case PROP_ENABLE_TAGS:
 	    gst->priv->enable_tags = g_value_get_boolean (value);
 	    break;
+	case PROP_VOLUME:
+	    parole_gst_set_volume (gst, g_value_get_double (value));
+	    break;
 	case PROP_CONF_OBJ:
 	    gst->priv->conf = g_value_get_pointer (value);
 	    
@@ -2159,6 +2164,29 @@ static void parole_gst_set_property (GObject *object,
     }
 }
 
+
+static gboolean
+parole_notify_volume_idle_cb (ParoleGst *gst)
+{
+    gdouble vol;
+
+    vol = gst_stream_volume_get_volume (GST_STREAM_VOLUME (gst->priv->playbin),
+                                      GST_STREAM_VOLUME_FORMAT_CUBIC);
+
+    gst->priv->volume = vol;
+    g_object_notify (G_OBJECT (gst), "volume");
+
+    return FALSE;
+}
+
+static void
+parole_notify_volume_cb (GObject             *object,
+		  GParamSpec          *pspec,
+		  ParoleGst *gst)
+{
+    g_idle_add ((GSourceFunc) parole_notify_volume_idle_cb, gst);
+}
+
 static void
 parole_gst_constructed (GObject *object)
 {
@@ -2190,6 +2218,18 @@ parole_gst_constructed (GObject *object)
 	g_error ("playbin load failed");
     }
     
+    gst->priv->audio_sink = gst_element_factory_make ("autoaudiosink", "audio");
+    if ( G_UNLIKELY (gst->priv->audio_sink == NULL) )
+	{
+	    GError *error;
+	    error = g_error_new (0, 0, "%s", _("Unable to load audio GStreamer plugin"
+					      ", check your GStreamer installation"));
+	    xfce_dialog_show_error (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gst))),
+				error, NULL);
+	    g_error_free (error);
+	    g_error ("autoaudiosink load failed");
+	}
+    
     if (enable_xv)
     {
 	gst->priv->video_sink = gst_element_factory_make ("xvimagesink", "video");
@@ -2216,6 +2256,7 @@ parole_gst_constructed (GObject *object)
     
     g_object_set (G_OBJECT (gst->priv->playbin),
 		  "video-sink", gst->priv->video_sink,
+		  "audio-sink", gst->priv->audio_sink,
 		  NULL);
     
     /*
@@ -2243,6 +2284,9 @@ parole_gst_constructed (GObject *object)
 
     g_signal_connect (gst->priv->playbin, "notify::source",
 		      G_CALLBACK (parole_gst_source_notify_cb), gst);
+    
+    g_signal_connect (gst->priv->playbin, "notify::volume",
+		      G_CALLBACK (parole_notify_volume_cb), gst);
 
 
     g_signal_connect (gst->priv->playbin, "about-to-finish",
@@ -2366,6 +2410,13 @@ parole_gst_class_init (ParoleGstClass *klass)
 							   G_PARAM_CONSTRUCT_ONLY|
 							   G_PARAM_READWRITE));
     
+    g_object_class_install_property (object_class,
+				     PROP_VOLUME,
+				     g_param_spec_double ("volume", NULL, NULL,
+							   0.0, 1.0, 0.0,
+	                                                   G_PARAM_READWRITE |
+                                                           G_PARAM_STATIC_STRINGS));
+    
     g_object_class_install_property (object_class,
 				     PROP_ENABLE_TAGS,
 				     g_param_spec_boolean ("tags",
@@ -2401,7 +2452,7 @@ parole_gst_init (ParoleGst *gst)
     gst->priv->scale_logo = TRUE;
     gst->priv->use_custom_subtitles = FALSE;
     gst->priv->custom_subtitles = NULL;
-    
+    gst->priv->volume = -1.0;
     gst->priv->conf = NULL;
     
     GTK_WIDGET_SET_FLAGS (GTK_WIDGET (gst), GTK_CAN_FOCUS);
@@ -2591,7 +2642,7 @@ void parole_gst_stop (ParoleGst *gst)
 void parole_gst_terminate (ParoleGst *gst)
 {
     gst->priv->terminating = TRUE;
-    parole_gst_terminate_internal (gst, TRUE);
+    parole_gst_terminate_internal (gst);
 }
 
 void parole_gst_shutdown (ParoleGst *gst)
@@ -2622,21 +2673,20 @@ void parole_gst_seek (ParoleGst *gst, gdouble seek)
 				       GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE));
 }
 
-void parole_gst_set_volume (ParoleGst *gst, gdouble value)
+void parole_gst_set_volume (ParoleGst *gst, gdouble volume)
 {
-    g_object_set (G_OBJECT (gst->priv->playbin),
-		  "volume", value,
-		  NULL);
+    gst_stream_volume_set_volume (GST_STREAM_VOLUME (gst->priv->playbin),
+				    GST_STREAM_VOLUME_FORMAT_CUBIC,
+				    volume);
+    volume = CLAMP (volume, 0.0, 1.0);
+    gst->priv->volume = volume;
+    
+    g_object_notify (G_OBJECT (gst), "volume");
 }
 						    
 gdouble	parole_gst_get_volume (ParoleGst *gst)
 {
-    gdouble volume;
-    
-    g_object_get (G_OBJECT (gst->priv->playbin),
-		  "volume", &volume,
-		  NULL);
-    return volume;
+    return gst->priv->volume;
 }
 
 ParoleState parole_gst_get_state (ParoleGst *gst)
