@@ -56,6 +56,8 @@ typedef struct
     
 } ParoleParserData;
 
+gchar *g_substr (const gchar* string, gint start, gint end);
+
 static gchar*
 parole_filename_to_utf8(const gchar* filename)
 {
@@ -306,12 +308,23 @@ out:
     return data.list;
 }
 
+gchar *
+g_substr (const gchar* string,
+          gint         start,
+          gint         end)
+{
+    gsize len = (end - start + 1);
+    gchar *output = g_malloc0 (len + 1);
+    return g_utf8_strncpy (output, &string[start], len);
+}
+
 static GSList *
 parole_pl_parser_parse_m3u (const gchar *filename)
 {
     GFile *file;
     gchar **lines;
     gchar *contents;
+    gchar *path, *pl_filename;
     GSList *list = NULL;
     gsize size;
     guint num_lines;
@@ -319,6 +332,7 @@ parole_pl_parser_parse_m3u (const gchar *filename)
     guint i;
     
     file = g_file_new_for_path (filename);
+    path = g_path_get_dirname(filename);
     
     if ( !g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) )
 	goto out;
@@ -351,17 +365,26 @@ parole_pl_parser_parse_m3u (const gchar *filename)
 
     for ( i = 0; lines[i] != NULL; i++)
     {
-	if ( lines[i][0] == '\0' || lines[i][0] == '#')
-	    continue;
+	    if ( lines[i][0] == '\0' || lines[i][0] == '#')
+	        continue;
+	    
+        if ( lines[i][0] == '/' ) {
+            pl_filename = g_strdup(lines[i]);
+        }
 
-	list = g_slist_append (list, parole_file_new (lines[i]));
+        else {
+	        pl_filename = g_strjoin("", path, lines[i], NULL);
+        }
+	    
+	    list = g_slist_append (list, parole_file_new (pl_filename));
     }
 
+    if (pl_filename)
+        g_free(pl_filename);
     g_strfreev (lines);
 out:
-
+    
     g_object_unref (file);
-
     return list;
 }
 
