@@ -201,6 +201,8 @@ struct ParoleMediaListPrivate
     GtkWidget *shuffle_button;
 	GtkWidget *settings_button;
 	GtkWidget *n_items;
+	
+	char *history[3];
 };
 
 enum
@@ -345,7 +347,7 @@ parole_media_list_files_open (ParoleMediaList *list, GSList *files, gboolean dis
 		  NULL);
     
     len = g_slist_length (files);
-    TRACE ("Adding files");
+    TRACE ("Adding %i files", len);
     
     if ( len > 1 )
         g_signal_emit (G_OBJECT (list), signals [SHOW_PLAYLIST], 0, TRUE);
@@ -525,12 +527,26 @@ void	parole_media_list_drag_data_received_cb (GtkWidget *widget,
 
 gboolean parole_media_list_key_press (GtkWidget *widget, GdkEventKey *ev, ParoleMediaList *list)
 {
-    if ( ev->keyval == GDK_Delete )
+    GtkWidget *vbox_player;
+    switch ( ev->keyval )
     {
-	parole_media_list_remove_clicked_cb (NULL, list);
-	return TRUE;
+        case GDK_Delete:
+            parole_media_list_remove_clicked_cb (NULL, list);
+            return TRUE;
+            break;
+        case GDK_Right:
+        case GDK_Left:
+        case GDK_Page_Down:
+        case GDK_Page_Up:
+        case GDK_Escape:
+            vbox_player = GTK_WIDGET(gtk_container_get_children( GTK_CONTAINER(gtk_widget_get_parent(gtk_widget_get_parent(gtk_widget_get_parent(gtk_widget_get_parent(gtk_widget_get_parent(gtk_widget_get_parent(widget))))))) )[0].data);
+            gtk_widget_grab_focus(vbox_player);
+            return TRUE;
+            break;
+        default:
+            return FALSE;
+            break;
     }
-    return FALSE;
 }
 
 /* Callback for the add button */
@@ -1791,19 +1807,36 @@ parole_media_list_add_by_path (ParoleMediaList *list, const gchar *path, gboolea
     GtkFileFilter *filter;
     guint len;
     gboolean ret = FALSE;
+    gchar *full_path;
     
     filter = parole_get_supported_media_filter ();
     g_object_ref_sink (filter);
     
-    TRACE ("Path=%s", path);
+    if (g_path_is_absolute(path)) {
+        full_path = g_strdup(path);
+    }
+    else 
+    {
+        if (g_file_test( g_strjoin("/", g_get_current_dir(), g_strdup(path), NULL), G_FILE_TEST_EXISTS))
+        {
+            full_path = g_strjoin("/", g_get_current_dir(), g_strdup(path), NULL);
+        }
+        else
+        {
+            full_path = g_strdup(path);
+        }
+    }
+    TRACE ("Path=%s", full_path);
     
-    parole_get_media_files (filter, path, TRUE, &files_list);
+    parole_get_media_files (filter, full_path, TRUE, &files_list);
     
     parole_media_list_files_open (list, files_list, FALSE, emit);
     
     len = g_slist_length (files_list);
     ret = len == 0 ? FALSE : TRUE;
     
+    g_free(full_path);
+    
     g_object_unref (filter);
     g_slist_free (files_list);
     return ret;
@@ -1909,8 +1942,10 @@ GtkTreeRowReference *parole_media_list_get_row_random (ParoleMediaList *list)
     GtkTreeRowReference *row = NULL;
     GtkTreeIter iter;
     GtkTreePath *path;
+    gchar *current_path;
     gchar *path_str;
     gint nch;
+    int count = 0;
 
     nch = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (list->priv->store), NULL);
     
@@ -1919,17 +1954,32 @@ GtkTreeRowReference *parole_media_list_get_row_random (ParoleMediaList *list)
 	return  NULL;
     }
     
-    path_str = g_strdup_printf ("%i", g_random_int_range (0, nch));
+    current_path = gtk_tree_path_to_string(gtk_tree_row_reference_get_path(parole_media_list_get_selected_row(list)));
+    path_str = g_strdup(current_path);
+    
+    if (!list->priv->history[0])
+        list->priv->history[0] = g_strdup(path_str);
+    
+    while (g_strcmp0(list->priv->history[0], path_str) == 0 || g_strcmp0(list->priv->history[1], path_str) == 0 || g_strcmp0(list->priv->history[2], path_str) == 0 || g_strcmp0(current_path, path_str) == 0)
+    {
+        path_str = g_strdup_printf ("%i", g_random_int_range (0, nch));
+        count += 1;
+        if (count >= 10 && g_strcmp0(current_path, path_str) != 0)
+            break;
+    }
+        
+    list->priv->history[2] = list->priv->history[1];
+    list->priv->history[1] = list->priv->history[0];
+    list->priv->history[0] = g_strdup(path_str);
     
     path = gtk_tree_path_new_from_string (path_str);
     g_free (path_str);
     
     if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path))
     {
-	row  = gtk_tree_row_reference_new (GTK_TREE_MODEL (list->priv->store), path);
-	//parole_media_list_select_path (list, path);
+    row  = gtk_tree_row_reference_new (GTK_TREE_MODEL (list->priv->store), path);
     }
-    
+
     gtk_tree_path_free (path);
     
     return row;
