@@ -26,7 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <parole/parole-provider-player.h>
+#include <src/misc/parole-provider-player.h>
 
 #include "parole-plugin-player.h"
 #include "parole-plugins-manager.h"
@@ -188,7 +188,7 @@ static void parole_plugin_player_iface_init (ParoleProviderPlayerIface *iface)
 
 static void 
 parole_plugin_player_media_state_changed_cb (ParoleGst *gst, const ParoleStream *stream, 
-					     ParoleMediaState state, ParolePluginPlayer *player)
+					     ParoleState state, ParolePluginPlayer *player)
 {
     g_signal_emit_by_name (G_OBJECT (player), "state-changed", stream, state);
 }
