@@ -31,8 +31,6 @@
 
 #include "notify-provider.h"
 
-#define RESOURCE_FILE 	"xfce4/src/misc/parole-plugins/notify.rc"
-
 static void   notify_provider_iface_init    (ParoleProviderPluginIface *iface);
 static void   notify_provider_finalize      (GObject                   *object);
 
@@ -46,6 +44,7 @@ struct _NotifyProvider
 {
     GObject                 parent;
     ParoleProviderPlayer   *player;
+    gchar                  *last_played_uri;
 
     NotifyNotification     *notification;
 };
@@ -85,7 +84,7 @@ notify_playing (NotifyProvider *notify, const ParoleStream *stream)
 {
     GdkPixbuf *pix;
     gboolean has_video;
-    gchar *title, *album, *artist, *year;
+    gchar *title, *album, *artist, *year, *stream_uri;
     gchar *message;
     ParoleMediaType media_type;
     
@@ -95,9 +94,16 @@ notify_playing (NotifyProvider *notify, const ParoleStream *stream)
 		  "artist", &artist,
 		  "year", &year,
 		  "has-video", &has_video,
-          "media-type", &media_type,		  
+          "media-type", &media_type,	
+          "uri", &stream_uri,  
 		  NULL);
 		  
+    if ( g_strcmp0(stream_uri, notify->last_played_uri) == 0 )
+    return;
+    
+    notify->last_played_uri = g_strdup(stream_uri);
+    g_free(stream_uri);
+		  
     if ( has_video )
     return;
 
